"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.accessSintegra = void 0;
const imageProcessing_1 = require("../helpers/imageProcessing");
const objectFormatting_1 = require("../helpers/objectFormatting");
const ocr_1 = require("../helpers/ocr");
const puppeteerHelper_1 = require("../helpers/puppeteerHelper");
const replaceColorsFromImage_1 = require("../helpers/replaceColorsFromImage");
const setupPuppeteer_1 = require("../helpers/setupPuppeteer");
const estados_1 = __importDefault(require("../utils/estados"));
const utils_1 = require("../utils/utils");
const NEXT_PAGE_CODE_SELECTOR = "span#ctl00_conteudoPaginaPlaceHolder_lblCodigoControleCertidao";
const SELECT_SELECTOR = "select#ctl00_conteudoPaginaPlaceHolder_filtroTabContainer_filtroEmitirCertidaoTabPanel_tipoFiltroDropDownList";
const INPUT_SELECTOR = "input#ctl00_conteudoPaginaPlaceHolder_filtroTabContainer_filtroEmitirCertidaoTabPanel_valorFiltroTextBox";
const CAPTCHA_IMG_SELECTOR = "img#ctl00_conteudoPaginaPlaceHolder_filtroTabContainer_filtroEmitirCertidaoTabPanel_imagemDinamica";
const ERROR_LABEL_SELECTOR = "span#ctl00_conteudoPaginaPlaceHolder_filtroTabContainer_filtroEmitirCertidaoTabPanel_MensagemErroFiltroLabel";
const CAPTCHA_INPUT_SELECTOR = "input#ctl00_conteudoPaginaPlaceHolder_filtroTabContainer_filtroEmitirCertidaoTabPanel_imagemDinamicaTextBox";
const RESULT_BUTTON_SELECTOR = "input#ctl00_conteudoPaginaPlaceHolder_filtroTabContainer_filtroEmitirCertidaoTabPanel_consultaPublicaButton";
const getTableSelectorBy = (nthOfType) => `table.consultaFiltrosTable table.consultaFiltrosTable table table table:nth-of-type(${nthOfType}) tr`;
const colorsToOverwrite = [
    { targetColor: "#000000", newColor: "#FFFFFF" },
    { targetColor: "#00FF00", newColor: "#FFFFFF" },
    { targetColor: "#00FFFF", newColor: "#FFFFFF" },
    { targetColor: "#FFFF00", newColor: "#FFFFFF" },
    { targetColor: "#FF00FF", newColor: "#FFFFFF" },
    { targetColor: "#0000FF", newColor: "#FFFFFF" },
    { targetColor: "#FF9F9E", newColor: "#FF0303" },
    { targetColor: "#FF6A6A", newColor: "#FF0303" },
    { targetColor: "#FFC6C6", newColor: "#FF0303" },
    { targetColor: "#FF0303", newColor: "#000000" },
];
async function accessSintegra(cnpj) {
    const browser = await setupPuppeteer_1.setupBrowser();
    try {
        const page = await setupPuppeteer_1.setupPage(estados_1.default.SAO_PAULO.url, browser);
        await setupInitialPart(page, cnpj);
        let text = await getErrorLabelText(page);
        while (text === "O texto digitado não confere com a imagem de segurança.") {
            await processCaptcha(page);
            text = await getErrorLabelText(page);
        }
        if (text ===
            "Não existem registros que atendem ao critério de filtro definido.") {
            throw new Error("Registro não existe");
        }
        await page.waitFor(NEXT_PAGE_CODE_SELECTOR);
        const sintegra = await retrieveAndProcessValues(page);
        return parseSintegraData(sintegra);
    }
    catch (err) {
        throw err;
    }
    finally {
        await browser.close();
    }
}
exports.accessSintegra = accessSintegra;
async function setupInitialPart(page, cnpj) {
    await page.waitFor(SELECT_SELECTOR);
    await page.select(SELECT_SELECTOR, "1");
    await page.waitForNavigation({ waitUntil: "networkidle0" });
    await page.waitFor(INPUT_SELECTOR);
    await page.type(INPUT_SELECTOR, cnpj);
    await processCaptcha(page);
}
async function processCaptcha(page) {
    const result = await processImage(page);
    await submitResult(page, result.data.text);
}
async function getErrorLabelText(page) {
    return await puppeteerHelper_1.extractText(page, ERROR_LABEL_SELECTOR);
}
function getSetOfText(selector, page) {
    return page.$$eval(selector, (elements) => elements.map((el) => {
        return [...el.children]
            .filter((el) => el.innerText.trim().length > 0)
            .map((el) => el.innerText);
    }));
}
async function retrieveAndProcessValues(page) {
    const firstSet = getSetOfText(getTableSelectorBy(2), page);
    const secondSet = getSetOfText(getTableSelectorBy(4), page);
    const thirdSet = getSetOfText(getTableSelectorBy(6), page);
    const values = await Promise.all([firstSet, secondSet, thirdSet]);
    return values.flat().reduce((acc, curr) => {
        if (curr.length > 0) {
            acc[objectFormatting_1.formatObjKey(curr[0])] = curr[1].trim();
        }
        if (curr.length === 3) {
            const splitVal = curr[2].split(":");
            acc[objectFormatting_1.formatObjKey(splitVal[0])] = splitVal[1].trim();
        }
        return acc;
    }, {});
}
async function processImage(page) {
    await page.waitFor(CAPTCHA_IMG_SELECTOR);
    const imgUrl = await page.$eval(CAPTCHA_IMG_SELECTOR, (node) => node === null || node === void 0 ? void 0 : node.src);
    const jimp = await imageProcessing_1.clearImage(imgUrl.toString());
    const clearedImg = await replaceColorsFromImage_1.removeColorsFromImage(await imageProcessing_1.getImageBuffer(jimp), colorsToOverwrite);
    return await ocr_1.processOcr(clearedImg);
}
async function submitResult(page, text) {
    await page.$eval(CAPTCHA_INPUT_SELECTOR, (el) => (el.value = ""));
    await page.type(CAPTCHA_INPUT_SELECTOR, ocr_1.formatOCRResult(text));
    await page.click(RESULT_BUTTON_SELECTOR);
}
function parseSintegraData(data) {
    if (utils_1.isObjectEmpty(data)) {
        throw new Error("No data");
    }
    const sintegra = {
        ...data,
        numero: data.n,
        municipio: data.municpio,
        situacaoCadastral: data.situaoCadastral,
        ocorrenciaFiscal: data.ocorrnciaFiscal,
        regimeDeApuracao: data.regimeDeApurao,
        atividadeEconomica: data.atividadeEconmica,
        dataDaSituacaoCadastral: data.dataDaSituaoCadastral,
    };
    delete sintegra.n;
    delete sintegra.municpio;
    delete sintegra.situaoCadastral;
    delete sintegra.ocorrnciaFiscal;
    delete sintegra.regimeDeApurao;
    delete sintegra.atividadeEconmica;
    delete sintegra.dataDaSituaoCadastral;
    return sintegra;
}
//# sourceMappingURL=sintegraSP.js.map